/****************************************************************************

	Copyright 1997, Adobe Systems Incorporated. All rights reserved.

    PROGRAM: DDEClien.c

    PURPOSE: Example of a standalone PageMaker Addition which sends
    	commands and queries thru DDE.

    FUNCTIONS:

    WinMain() - calls initialization function, processes message loop
    InitApplication() - initializes window data and registers window
    InitInstance() - saves instance handle and creates main window
    MainWndProc() - processes messages
    About() - processes messages for "About" dialog box

    COMMENTS:

	Windows can have several copies of your application running at the
	same time.  The variable ghInst keeps track of which instance this
	application is so that processing will be to the correct window.

****************************************************************************/

#include <windows.h>            // required for all Windows applications 
#include <ddeml.h>
#include "ddeclient.h"
#include <stdio.h>
#include <string.h>          

HANDLE ghInst;               // current instance              

HWND   ghWnd;               // Main window handle.                

/****************************************************************************

    FUNCTION: WinMain(HINSTANCE, HINSTANCE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point
        for the program.  This function calls the application initialization
        routine, if no other instance of the program is running, and always
        calls the instance initialization routine.  It then executes a message
        retrieval and dispatch loop that is the top-level control structure
        for the remainder of execution.  The loop is terminated when a WM_QUIT
        message is received, at which time this function exits the application
        instance by returning the value passed by PostQuitMessage().

        If this function must abort before entering the message loop, it
        returns the conventional value NULL.

****************************************************************************/

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;                     // message              

    if (!hPrevInstance)          // Other instances of app running? 
    if (!InitApplication(hInstance)) // Initialize shared things 
        return (FALSE);      // Exits if unable to initialize     

    // Perform initializations that apply to a specific instance 

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

	PostMessage(ghWnd,WM_COMMAND,IDM_DISCONNECT,0); // set initial state of menu

    // Acquire and dispatch messages until a WM_QUIT message is received. 
	
    while (GetMessage(&msg,    // message structure              
        NULL,          // handle of window receiving the message 
        (unsigned int)NULL,          // lowest message to examine          
        (unsigned int)NULL))         // highest message to examine         
    {
    TranslateMessage(&msg);    // Translates virtual key codes       
    DispatchMessage(&msg);     // Dispatches message to window       
    }
    return (msg.wParam);       // Returns the value from PostQuitMessage 
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other
        instances of the application are running.  This function performs
        initialization tasks that can be done once for any number of running
        instances.

        In this case, we initialize a window class by filling out a data
        structure of type WNDCLASS and calling the Windows RegisterClass()
        function.  Since all instances of this application use the same window
        class, we only need to do this when the first instance is initialized.


****************************************************************************/

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the       
    // main window.                                                           

    wc.style = (unsigned int)NULL;                    // Class style(s).                    
    wc.lpfnWndProc = MainWndProc;       // Function to retrieve messages for  
                                        // windows of this class.             
    wc.cbClsExtra = 0;                  // No per-class extra data.           
    wc.cbWndExtra = 0;                  // No per-window extra data.          
    wc.hInstance = hInstance;           // Application that owns the class.   
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(GRAY_BRUSH);
    wc.lpszMenuName =  "DDEClientMenu";   // Name of menu resource in .RC file. 
    wc.lpszClassName = "DDEClientWClass"; // Name used in call to CreateWindow. 

    // Register the window class and return success/failure code. 

    return (RegisterClass(&wc));

}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

	This function is called at initialization time for every instance of
	this application.  This function performs initialization tasks that
	cannot be shared by multiple instances.

	In this case, we save the instance handle in a static variable and
	create and display the main program window.

****************************************************************************/


BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{

    // Save the instance handle in static variable, which will be used in  
    // many subsequence calls from this application to Windows.            

    ghInst = hInstance;

    // Create a main window for this application instance.  

    ghWnd = CreateWindow(
	"DDEClientWClass",                 // See RegisterClass() call.          
	"DDE Client Sample Application",   // Text for window title bar.         
	WS_OVERLAPPEDWINDOW,            // Window style.                      
	CW_USEDEFAULT,                  // Default horizontal position.       
	CW_USEDEFAULT,                  // Default vertical position.         
	CW_USEDEFAULT,                  // Default width.                     
	CW_USEDEFAULT,                  // Default height.                    
	NULL,                           // Overlapped windows have no parent. 
	NULL,                           // Use the window class menu.         
	hInstance,                      // This instance owns this window.    
	NULL                            // Pointer not needed.                
    );

    // If window could not be created, return "failure" 

    if (!ghWnd)
	return (FALSE);

    // Make the window visible; update its client area; and return "success" 

    ShowWindow(ghWnd, nCmdShow);  // Show the window                        
    UpdateWindow(ghWnd);          // Sends WM_PAINT message                 
    return (TRUE);               // Returns the value from PostQuitMessage 

}

// This is DDE stuff 

/*************************** DDE globals ***********************************/

HSZ ghszServeName;
HSZ ghszSystemTopic;

HCONV ghConv;

HSZ ghszCommand;
HSZ ghszQuery;
HDDEDATA ghReply;

// Maximal length of reply is hardcoded here
// For real additions change MAX_REPLY_LEN or
// dynamically allocate storage for reply.

#define MAX_REPLY_LEN 128
char gReply[MAX_REPLY_LEN];

DWORD gidInst;
HDDEDATA EXPENTRY DdeCallBack(WORD wType, WORD wFmt, HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1, DWORD dwData2)
{ switch(wType)
  { case XTYP_DISCONNECT:
  			EnableMenuItem(GetMenu(ghWnd),IDM_DDECONNECT,MF_ENABLED);
			EnableMenuItem(GetMenu(ghWnd),IDM_COMMAND,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_QUERY,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_SCRIPT,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_DISCONNECT,MF_DISABLED | MF_GRAYED);

     		MessageBox(ghWnd,(LPSTR)"PM disconnected","",MB_OK);
     break;
  }

  return (HDDEDATA) NULL;
}

FARPROC lpDdeProc;


/****************************************************************************

    FUNCTION: MainWndProc(hWnd, UINT, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

    WM_COMMAND    - application menu (About dialog box)
    WM_DESTROY    - destroy window

    COMMENTS:

    To process the IDM_ABOUT message, call MakeProcInstance() to get the
    current instance address of the About() function.  Then call Dialog
    box which will create the box according to the information in your
    ddeclien.rc file and turn control over to the About() function.  When
    it returns, free the intance address.

****************************************************************************/

long CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{   
    switch (message)
    {
	case WM_COMMAND:       // message: command from application menu 
	    switch (wParam)
	    {   
	    	case IDM_ABOUT:
				DialogBox(ghInst,    // current instance          
		    	"AboutBox",         // resource to use           
		    	ghWnd,               // parent handle             
		    	About);             // About() instance address  
		break;

		case IDM_DDECONNECT:
			lpDdeProc = MakeProcInstance((FARPROC) DdeCallBack, ghInst);
			if (DdeInitialize(&gidInst,(PFNCALLBACK)lpDdeProc, CBF_FAIL_EXECUTES|CBF_FAIL_POKES, 0L)!=DMLERR_NO_ERROR)
			{ 
				MessageBox(NULL,"Initialization failed","",MB_OK); return 0L;
			}
			ghszServeName = DdeCreateStringHandle(gidInst,"PageMaker",CP_WINANSI);
			ghszSystemTopic=DdeCreateStringHandle(gidInst,"Test",CP_WINANSI);
			if (!(ghConv=DdeConnect(gidInst,ghszServeName,ghszSystemTopic,(PCONVCONTEXT)NULL)))
			{ 
				MessageBox(NULL,"PageMaker does not seem to be running...","",MB_OK); return 0L;
			}
			EnableMenuItem(GetMenu(ghWnd),IDM_DDECONNECT,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_COMMAND,MF_ENABLED);
			EnableMenuItem(GetMenu(ghWnd),IDM_QUERY,MF_ENABLED);
			EnableMenuItem(GetMenu(ghWnd),IDM_SCRIPT,MF_ENABLED);
			EnableMenuItem(GetMenu(ghWnd),IDM_DISCONNECT,MF_ENABLED);
				
			break;

		case IDM_COMMAND:
			if (IDCANCEL==MessageBox(NULL,"Going to open new default pub by script command 'new'","",MB_OKCANCEL))
				break;
			ghszCommand = DdeCreateStringHandle(gidInst,"paste",CP_WINANSI);
			DdeClientTransaction(NULL,(DWORD)0,ghConv,ghszCommand,CF_TEXT,XTYP_REQUEST,(DWORD)TIMEOUT_ASYNC,NULL);
			break;

		case IDM_QUERY:
			if (IDCANCEL==MessageBox(NULL,"Going to query selectlist","",MB_OKCANCEL))
				break;
			ghszQuery = DdeCreateStringHandle(gidInst,"getselectlist",CP_WINANSI);
			ghReply = DdeClientTransaction(NULL,0,ghConv,ghszQuery,CF_TEXT,XTYP_REQUEST,1000,NULL);
			DdeGetData(ghReply,gReply,(long)(MAX_REPLY_LEN-1),0L);
			DdeFreeStringHandle(gidInst,ghszQuery);
			MessageBox(NULL,gReply,"select list is:",MB_OK);
			break;

		case IDM_SCRIPT:
			{
			char x[10],y[10], command[20];  // x,y - height,width of a page
			if (IDCANCEL==MessageBox(NULL,"Going to change page orientation by the following script:\ngetpagesize =>x,y\npagesize y,x","",MB_OKCANCEL))
				break;
			ghszQuery = DdeCreateStringHandle(gidInst,"getpagesize",CP_WINANSI);
			ghReply = DdeClientTransaction(NULL,0,ghConv,ghszQuery,CF_TEXT,XTYP_REQUEST,1000,NULL);
			if (!ghReply) 
			{
				MessageBox(NULL,"PageMaker sent empty reply.\nAre you sure your pub is opened?","",MB_OK);
				DdeFreeStringHandle(gidInst,ghszQuery);
				break;
			}
			DdeGetData(ghReply,gReply,(long)(MAX_REPLY_LEN-1),0L);
			DdeFreeStringHandle(gidInst,ghszQuery);
			// now swap height and width of a page
			sscanf(gReply,"%[^,]",x);
			sscanf(gReply+strlen(x)+1,"%s",y);
			sprintf(command,"pagesize %s,%s",y,x);
			ghszCommand = DdeCreateStringHandle(gidInst,command,CP_WINANSI);
			DdeClientTransaction(NULL,(DWORD)0,ghConv,ghszCommand,CF_TEXT,XTYP_REQUEST,(DWORD)TIMEOUT_ASYNC,NULL);
			break;
			}
			
		case IDM_DISCONNECT:
			DdeFreeStringHandle(gidInst,ghszServeName);
			DdeFreeStringHandle(gidInst,ghszSystemTopic);
			DdeDisconnect(ghConv);
			EnableMenuItem(GetMenu(ghWnd),IDM_DDECONNECT,MF_ENABLED);
			EnableMenuItem(GetMenu(ghWnd),IDM_COMMAND,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_QUERY,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_SCRIPT,MF_DISABLED | MF_GRAYED);
			EnableMenuItem(GetMenu(ghWnd),IDM_DISCONNECT,MF_DISABLED | MF_GRAYED);
			break;
	    }
	    break;
	case WM_DESTROY:          // message: window being destroyed 
		DdeDisconnect(ghConv); // say goodby to PageMaker 
	    PostQuitMessage(0);
	    break;

	default:                  // Passes it on if unproccessed    
	    return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return ((long)NULL);
}


/****************************************************************************

    FUNCTION: About(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

    COMMENTS:

    No initialization is needed for this dialog box, but TRUE
    must be returned to Windows.

    Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
	case WM_INITDIALOG:            // message: initialize dialog box 
	    return (TRUE);

	case WM_COMMAND:               // message: received a command 
	    if (wParam == IDOK         // "OK" box selected?          
		|| wParam == IDCANCEL) // System menu close command?  
	    {
		EndDialog(hDlg, TRUE); // Exits the dialog box        
		return (TRUE);
	    }
	    break;
    }
    return (FALSE);               // Didn't process a message    
}
