/*
 *--- FontPalette.cpp ---------------------------------------------
 * Creates a fonts palette in PageMaker, using the IAdobeVector interface.
 *
 *
 *
 * Copyright 1997, Adobe Systems Incorporated.  All rights reserved.
 * 
 * 
 *
 *------------------------------------------------------------------
 */

#include <string.h>
#include <objbase.h>	//required for IAdobeVector
#include <initguid.h>	//required for IAdobeVector

#include "PMPlugin.h"
#include "CIWindow.h"
#include "CICommandsAndQueries.h"
#include "CIInterfaceManager.h"
#include "CIBasic.h"
#include "CIObjectAccess.h"
#include "IAdobeVector.h"
#include "PMInterfaceIDs.h"
#include "PMCQErrs.h"
#include "PMEvent.h"

#include "FontPalette.h"
#include "resource.h"

#include "PCL.h"

/*---externals ---*/
extern PMMessage			*gPMMessage;
extern sPMParamBlockPtr		gPB;
extern CIInterfaceManager	*gInterfaceMgr;
extern CICommandQuery		*gCmdQry;
extern HWND					hSubWindow;
CIPMWindow					*thePalette = NULL;

/*
 *--- ColorerPlugin ------------------------------------------------
 * No special initialization needed.
 *------------------------------------------------------------------
 */
FontPalette::FontPalette() : PPluginCall()
{

}

void FontPalette::DoLoad()
{
// PageMaker will call this function each time your plug-in is loaded.
//	On load, we acquire the CICommandQuery Interface, and get the 
//	Parameter block, this saves the overhead of loading it each time 
//	a command is used, and ensures that we have the Parameter block
//	if the plug-in exports interfaces.
	gInterfaceMgr->AcquirePMInterface(PMIID_CMDQRY,  (void **)&gCmdQry);
	gCmdQry->Retrieve(&gPB);
	
}

void FontPalette::DoInvoke()
{
// DoInvoke is called when your plug-in is launched from the 'Plug-ins' menu

}

void FontPalette::DoUnload()
{
//When your plug-in is going to be unloaded, this function is called.
// PageMaker may load and unload your plug-in several times during a session if 
// the plug-in does not have the 'stay in memory' attribute set.
	gInterfaceMgr->ReleasePMInterface( gCmdQry );
	gCmdQry = NULL;
	gPB = NULL;
}

void FontPalette::DoRegister()
{
// This is where you register your plug-in to receive event notification,
// DoRegister is also the first opportunity to add interfaces to PageMaker.
	CIBasic *basic = NULL;

// At registration time, the application is not fully initialized,
//	Since we will use commands and queries in creating the palette, we must
//	wait until the application has finished its startup routines, fortunately
//	we can register for the PMEVT_APP_STARTUP_AFTER event.
	gInterfaceMgr->AcquirePMInterface( (unsigned long)PMIID_BASIC, (void **)&basic );
	basic->RegisterPMEvent( (PMEventID) PMEVT_APP_STARTUP_AFTER );
	basic->RegisterPMEvent( (PMEventID) PMEVT_SELECTOBJECT_AFTER );
	gInterfaceMgr->ReleasePMInterface( basic );


}

void FontPalette::DoEvent()
{
// This is where you handle the PageMaker events that you have requested
// notification for.
	PMEvent			*event;
	PMErr			errorCheck;
	CIPMWindow		*myWindow = NULL;
	CIPMWinStyle	*winAttribs = NULL;
	CIBasic			*basic = NULL;
	int				nCount;
	
	event = (PMEvent *) gPMMessage->pPMData;
	
	if ( event->eventID == PMEVT_APP_STARTUP_AFTER )
	{
		
		// First, acquire the CIPMWinStyle and CIPMWindow interfaces.	
		errorCheck = gInterfaceMgr->AcquirePMInterface( PMIID_WINSTYLE, (void **)&winAttribs );
		if ( errorCheck != CQ_SUCCESS )
		{
			throw errorCheck;
		}
		errorCheck = gInterfaceMgr->AcquirePMInterface( PMIID_WINDOW, (void **)&myWindow );
		if ( errorCheck != CQ_SUCCESS )
		{
			throw errorCheck;
		}
		
		// Our palette will be a fixed size, and set to 
		// stay on top. 
		winAttribs->SetBorderState(FALSE);
		winAttribs->SetVisibilityState(FALSE);
		winAttribs->SetTitleBarState(TRUE);
		winAttribs->SetZoomState(FALSE);
		winAttribs->SetSizeState(FALSE);
		winAttribs->SetCloseState(TRUE);
		winAttribs->SetMaximizeState(FALSE);
		winAttribs->SetStayOnTopState(TRUE);
		winAttribs->SetModalState(FALSE);
		winAttribs->SetDefaultWindowState(TRUE);
		
		// We will create the window, and leave it hidden.
		myWindow->Create(winAttribs, (void *)2);
		
		// We are done with the CIPMWinStyle interface and so we can
		//	release it now. We have to keep the pointer to the 
		//	CIPMWindow interface around as long as we are using the 
		//	palette.
		gInterfaceMgr->ReleasePMInterface( winAttribs );
		myWindow->AddToMenu("Show font info palette", "Hide font info palette", kAdnLayoutState);

		thePalette = myWindow;

		gInterfaceMgr->AcquirePMInterface( (unsigned long)PMIID_BASIC, (void **)&basic);
		basic->UnregisterPMEvent( PMEVT_APP_STARTUP_AFTER );
		gInterfaceMgr->ReleasePMInterface( basic );

		//The rest of  the work in creating the palette is done in response to
		// the WM_CREATE message on the Windows platforms, or in the InitMyPalette
		// method on the Macintosh platforms.
#ifdef MACINTOSH
		InitMyPalette();
#endif // Macintosh


	}

	if ( event->eventID == PMEVT_SELECTOBJECT_AFTER )
	{
		//One or more objects have been selected, we check to make sure only 
		// one is selected and then we add it's name & type in the palette.
		// If the object has fonts, we will also add the font names to 
		// the list.
		
		PMOBJ_REC objRec;
		IAdobeVector *ptAV;
		CIObjectAccess *pObjAcc = NULL;
		char szFontName[256];
		short i;
		PMObjAttr tobjAttr;

		PGetSelectIDList objectSelectedList;

		if(objectSelectedList.Count() == 1)
		{
			// This palette only works with one object selected.
			PGetLinkInfo linkInfo;
			char szDrive[_MAX_DRIVE], szDir[_MAX_DIR], szFName[_MAX_FNAME], szExt[_MAX_EXT];
			char szShortName[_MAX_PATH];

			_splitpath(linkInfo.fFilename, szDrive, szDir, szFName, szExt);

			strcpy(szShortName, szFName);
			if(szExt[0] != '.')
			{
				strcat(szShortName, ".");
			}

			strcat(szShortName, szExt);

			// Add the file name and file type to the palette.

			SetDlgItemText(hSubWindow, IDC_ITEM, szShortName);
			SetDlgItemText(hSubWindow, IDC_TYPE, linkInfo.sKind);

			if(gInterfaceMgr->AcquirePMInterface(PMIID_OBJACC, (LPVOID *)&pObjAcc))
			{	// We need acces to the object access interface (CIObjectAccess.h)
				return;
			}

			SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_RESETCONTENT, 0, 0);

			pObjAcc->GetPMObjRecFromObjID(objectSelectedList.nObjectID, &objRec);
			pObjAcc->GetObjAttr(&objRec, &tobjAttr);
			
			// Here's where we start the real 'IAdobeVector' code.
			//	Since the IAdobeVector interface is really an OLE Interface, we
			//	have to do an OleQueryInterface, which is part of CIObjectAccess
			if(pObjAcc->OleQueryInterface(&objRec, (void *)&IID_IAdobeVector, (void **)&ptAV) != AV_SUCCESS)
			{
				SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_ADDSTRING, 0, (LPARAM)"None found");
				return;	
			}
			
			i = 0;
			
				// The IAdobeInterface has FontNeeded and FontSupplied methods,
				//	see the documentation for the full list of methods.
			while(ptAV->FontNeeded(i, szFontName, 256) == AV_SUCCESS)
			{

				strcat(szFontName, "\tN");

				if(SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_FINDSTRINGEXACT, -1, (LPARAM)szFontName) == LB_ERR)
				{
					SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_ADDSTRING, 0, (LPARAM)szFontName);
				}
				i++;
			}

			i = 0;

			while(ptAV->FontSupplied(i, szFontName, 256) == AV_SUCCESS)
			{

				strcat(szFontName, "\tY");
				if(SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_FINDSTRINGEXACT, -1, (LPARAM)szFontName) == LB_ERR)
				{
					SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_ADDSTRING, 0, (LPARAM)szFontName);
				}
				i++;
			}
			ptAV->Release();
			if((nCount = SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_GETCOUNT, 0, 0)) != LB_ERR)
			{
				if(nCount == 0)
				{
					SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_ADDSTRING, 0, (LPARAM)"None found");
				}
			}

		}
		else
		{
			// Nothing is selected 
			SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_RESETCONTENT, 0, 0);
			SendDlgItemMessage(hSubWindow, IDC_FACE_LIST, LB_ADDSTRING, 0, (LPARAM)"More than one object");
		}
		gInterfaceMgr->ReleasePMInterface((LPVOID)pObjAcc);
	}

}

void FontPalette::DoSysEvent()
{
// PageMaker will pass system events for windows that you create in your plug-in
	PaletteEvents();
}

void FontPalette::DoAcquireInterface()
{
// If you've added an interface to PageMaker, then this function is called when
// another plug-in attempts to acquire the interface.

}

void FontPalette::DoReleaseInterface()
{
// The interface that you have added to PageMaker has been released by another 
// plug-in. 

}


void FontPalette::DoShutdown()
{
// The application is shutting down, and this is your last chance to clean things
// up, write preferences to disk, etc.
	thePalette->Destroy();
}
/*
int ConvertX( int x )
{
	int nDlgBaseX;
	LONG lBase = GetDialogBaseUnits();

	nDlgBaseX = LOWORD( lBase );

	return x * nDlgBaseX / 4;
}

int ConvertY( int y )
{
	int nDlgBaseY;

	LONG lBase = GetDialogBaseUnits();

	nDlgBaseY = HIWORD( lBase );

	return y * nDlgBaseY / 8;
}
*/